"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var React = require("react");
var _ = require("underscore");
var moment = require("moment");
var classNames = require("classnames");
var formBinders_1 = require("../../form/formBinders");
var grid_1 = require("./../../layout/grid");
var icons_1 = require('./../../../utilities/icons');
var icon_1 = require('./../../display/icon');
var isoFormat = "YYYY-MM-DD";
var CalendarInput = (function (_super) {
    __extends(CalendarInput, _super);
    function CalendarInput(props) {
        _super.call(this, props);
        this.mouseDownOnCalendar = false;
        var todaysDate = moment().startOf('day');
        this.state = { displayedDate: todaysDate.clone(), selectedDate: todaysDate, pickerBodyVisible: false, showOnTop: false, calendarOffset: 0 };
        moment.locale(props.locale);
        var seed = Math.random().toFixed(4);
        this.inputElementId = "date-picker-text-input-" + seed;
        this.bodyElementId = "date-picker-body-" + seed;
    }
    CalendarInput.prototype.onDaySelected = function (date) {
        if (!this.fallsWithinRange(date)) {
            return;
        }
        var newDate = date.clone();
        if (this.inputElement) {
            this.inputElement.value = newDate.format(this.format);
        }
        this.setState({ selectedDate: newDate, displayedDate: newDate.clone(), pickerBodyVisible: false });
        if (this.props.onDateChanged) {
            this.props.onDateChanged(this.props.returnString ? newDate.format(isoFormat) : newDate.clone());
        }
    };
    CalendarInput.prototype.isEndOfMonth = function (date) {
        var endOfMonth = date.clone().endOf('month');
        return endOfMonth.isSame(date, 'day');
    };
    CalendarInput.prototype.fallsWithinRange = function (date) {
        if (this.props.min && date.isBefore(this.getMinMaxAsMoment(this.props.min), 'day')) {
            return false;
        }
        if (this.props.max && date.isAfter(this.getMinMaxAsMoment(this.props.max), 'day')) {
            return false;
        }
        return true;
    };
    CalendarInput.prototype.calcTop = function () {
        if (this.inputElement) {
            var bounds = this.inputElement.getBoundingClientRect();
            this.setState({ calendarOffset: bounds.bottom });
        }
    };
    CalendarInput.prototype.getDaysInMonth = function () {
        var days = [];
        var a = this.state.displayedDate.clone().startOf('month').startOf('day');
        var b = a.clone().endOf('month');
        var firstDay = false;
        for (var m = moment(a); m.isBefore(b); m.add(1, 'days')) {
            if (!firstDay) {
                firstDay = true;
                var firstDayIndex = m.weekday();
                for (var i = firstDayIndex; i > 0; i--) {
                    days.push(this.getDayComponent(true, this.onDaySelected.bind(this), m.clone().subtract(i, 'days')));
                }
            }
            days.push(this.getDayComponent(false, this.onDaySelected.bind(this), m.clone()));
            if (this.isEndOfMonth(m)) {
                var lastDayIndex = m.weekday();
                for (var i = 1; i < 7 - lastDayIndex; i++) {
                    days.push(this.getDayComponent(true, this.onDaySelected.bind(this), m.clone().add(i, 'days')));
                }
            }
        }
        return days;
    };
    CalendarInput.prototype.getDayComponent = function (notInCurrentMonth, dayClicked, date) {
        var d = date.clone();
        var dateWithinRange = this.fallsWithinRange(d);
        var isSelected = d.isSame(this.state.selectedDate, 'day');
        return React.createElement(CalendarDay, {key: "Calendar_day_" + date.format('DDMMYYYY'), selected: isSelected, withinRange: dateWithinRange, notInCurrentMonth: notInCurrentMonth, dayClicked: dayClicked, date: d});
    };
    CalendarInput.prototype.changeMonth = function (increment) {
        var _this = this;
        this.setState({ displayedDate: this.state.displayedDate.add(increment, 'months') }, function () {
            _this.shouldShowOnTop();
        });
    };
    CalendarInput.prototype.checkDate = function (dateString) {
        var m = moment(dateString, this.format, false);
        if (m.isValid() && this.fallsWithinRange(m)) {
            var formattedDate = m.format(this.format);
            if (this.inputElement) {
                this.inputElement.value = formattedDate;
            }
            this.setState({ selectedDate: m.clone(), displayedDate: m.clone() });
            if (this.props.onDateChanged) {
                this.props.onDateChanged(this.props.returnString ? m.format(isoFormat) : m.clone());
            }
        }
        else {
            if (this.inputElement) {
                this.inputElement.value = this.state.selectedDate.format(this.format);
            }
        }
    };
    CalendarInput.prototype.componentWillMount = function () {
        this.format = this.props.nativeInput ? isoFormat : this.props.format;
        if (this.props.date) {
            if (typeof this.props.date === "string") {
                var mDate = moment(this.props.date, isoFormat);
                this.setState({ selectedDate: mDate, displayedDate: mDate.clone() });
            }
            else {
                var pDate = moment(this.props.date).startOf('day');
                pDate.locale(this.props.locale);
                this.setState({ selectedDate: pDate, displayedDate: pDate.clone() });
            }
        }
    };
    CalendarInput.prototype.componentWillUnmount = function () {
        if (!this.props.nativeInput) {
            window.removeEventListener('mousedown', this);
        }
    };
    CalendarInput.prototype.componentDidMount = function () {
        this.inputElement = document.getElementById(this.inputElementId);
        this.bodyElement = document.getElementById(this.bodyElementId);
        if (!this.props.nativeInput) {
            window.addEventListener('mousedown', this);
        }
    };
    CalendarInput.prototype.handleEvent = function (e) {
        if (this.mouseDownOnCalendar && e.type !== "mousewheel") {
            return;
        }
        document.removeEventListener("mousewheel", this, false);
        this.setState({ pickerBodyVisible: false });
        this.inputElement.blur();
    };
    CalendarInput.prototype.onInputFocus = function () {
        var _this = this;
        document.addEventListener("mousewheel", this, false);
        this.calcTop();
        this.setState({ pickerBodyVisible: true }, function () {
            _this.shouldShowOnTop();
        });
    };
    CalendarInput.prototype.shouldShowOnTop = function () {
        if (this.props.nativeInput || !this.inputElement) {
            return;
        }
        var height = this.bodyElement.clientHeight + 50;
        var visibleBottom = (window.innerHeight + window.scrollY);
        var inputRect = this.inputElement.getBoundingClientRect();
        var remainingSpace = window.innerHeight - inputRect.bottom;
        console.log("height: " + height + ", window height " + window.innerHeight + ", remainingSpace " + remainingSpace);
        if (remainingSpace < height) {
            this.setState({ showOnTop: true });
            return true;
        }
        else {
            this.setState({ showOnTop: false });
            return false;
        }
    };
    CalendarInput.prototype.getMinMaxAsMoment = function (input) {
        if (typeof input === "string") {
            return moment(input, isoFormat, true);
        }
        else {
            return input;
        }
    };
    CalendarInput.prototype.getMinMaxAsString = function (input) {
        if (typeof input === "string") {
            return input;
        }
        else {
            return input.format(isoFormat);
        }
    };
    CalendarInput.prototype.render = function () {
        var _this = this;
        var weekdays = _.range(0, 7).map(function (n) { return React.createElement("div", {className: "date-picker-week-day", key: "day_name_" + n}, moment().startOf('week').add(n, 'days').format('dd')); });
        var days = this.getDaysInMonth();
        var currentDisplayDate = this.state.displayedDate.format("MMMM - YYYY");
        var classes = classNames("date-picker-body", {
            "date-picker-body-visible": this.state.pickerBodyVisible && !this.props.alwaysShowCalendar,
            "date-picker-top": this.state.showOnTop,
            "always-show-calendar": this.props.alwaysShowCalendar
        });
        var rootClasses = classNames("date-picker", this.props.className, {
            "has-icon": this.props.icon !== null,
            "disabled": this.props.disabled
        });
        if (this.props.nativeInput) {
            return (React.createElement("div", {className: rootClasses}, this.props.icon && React.createElement(icon_1.Icon, {icon: this.props.icon}), React.createElement("input", {id: this.inputElementId, type: "date", min: this.props.min ? this.getMinMaxAsString(this.props.min) : '', max: this.props.max ? this.getMinMaxAsString(this.props.max) : '', onChange: function (e) { return _this.checkDate(e.target.value); }, defaultValue: this.state.selectedDate.format(this.format), onBlur: function (e) { return _this.checkDate(e.target.value); }})));
        }
        return (React.createElement("div", {className: rootClasses, onMouseDown: function () { return _this.mouseDownOnCalendar = true; }, onMouseUp: function () { return _this.mouseDownOnCalendar = false; }}, React.createElement(icon_1.Icon, {icon: this.props.icon || icons_1.Icons.Icomoon.calendar2}), !this.props.alwaysShowCalendar &&
            React.createElement("input", {id: this.inputElementId, disabled: this.props.disabled, type: "text", defaultValue: this.state.selectedDate.format(this.format), onBlur: function (e) { return _this.checkDate(e.target.value); }, onFocus: function (e) { return _this.onInputFocus(); }}), React.createElement("div", {id: this.bodyElementId, className: classes, style: { top: this.state.calendarOffset + "px" }}, React.createElement("div", {className: "date-picker-body-wrapper"}, React.createElement(grid_1.Grid, {className: "date-picker-header"}, React.createElement(grid_1.Row, null, React.createElement(grid_1.Col, {onClick: function () { return _this.changeMonth(-1); }, width: "auto"}, "<"), React.createElement(grid_1.Col, null, currentDisplayDate), React.createElement(grid_1.Col, {onClick: function () { return _this.changeMonth(1); }, width: "auto"}, ">"))), React.createElement("div", {className: "date-picker-days"}, weekdays, days)))));
    };
    CalendarInput.Icomoon = icons_1.Icons.Icomoon;
    CalendarInput.defaultProps = {
        format: 'DD/MM/YYYY',
        date: moment().startOf('day'),
        locale: 'en-gb'
    };
    return CalendarInput;
}(React.Component));
exports.CalendarInput = CalendarInput;
var CalendarDay = (function (_super) {
    __extends(CalendarDay, _super);
    function CalendarDay() {
        _super.apply(this, arguments);
    }
    CalendarDay.prototype.render = function () {
        var _this = this;
        var classes = classNames({
            "not-in-month": this.props.notInCurrentMonth,
            "selected-day": this.props.selected,
            "day-disabled": !this.props.withinRange
        });
        return React.createElement("div", {className: classes, onClick: function () { return _this.props.dayClicked(_this.props.date); }}, this.props.date.format('DD'));
    };
    return CalendarDay;
}(React.Component));
var CalendarInputFormBinder = (function (_super) {
    __extends(CalendarInputFormBinder, _super);
    function CalendarInputFormBinder() {
        _super.apply(this, arguments);
    }
    CalendarInputFormBinder.customValue = function (dataName) {
        return new CalendarInputFormBinder(dataName, "date");
    };
    CalendarInputFormBinder.prototype.handleValueChanged = function (props, dataBinder, notifyChanged) {
        var _this = this;
        props.onDateChanged = function (e) {
            if (typeof e === 'string') {
                _this.onChanged(dataBinder, e, notifyChanged);
            }
            else {
                _this.onChanged(dataBinder, e.format("YYYY-MM-DD"), notifyChanged);
            }
        };
    };
    return CalendarInputFormBinder;
}(formBinders_1.FormBinderBase));
exports.CalendarInputFormBinder = CalendarInputFormBinder;
